/****************************************************************************
 *                                                                          *
 * File    : main.c                                                         *
 *                                                                          *
 * Purpose : Generic dialog based Win32 application.                        *
 *                                                                          *
 * History : Date      Reason                                               *
 *           00/00/00  Created                                              *
 *                                                                          *
 ****************************************************************************/

/* 
 * Either define WIN32_LEAN_AND_MEAN, or one or more of NOCRYPT,
 * NOSERVICE, NOMCX and NOIME, to decrease compile time (if you
 * don't need these defines -- see windows.h).
 */

#define WIN32_LEAN_AND_MEAN
/* #define NOCRYPT */
/* #define NOSERVICE */
/* #define NOMCX */
/* #define NOIME */

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <commdlg.h>
#include <tchar.h>
#include <stdio.h>
#include <stdlib.h>

#include "main.h"
#include "DataGridView.h"

#define NELEMS(a)  (sizeof(a) / sizeof((a)[0]))

#define Refresh(A) RedrawWindow(A,NULL,NULL,RDW_ERASE|RDW_INVALIDATE|RDW_ALLCHILDREN|RDW_UPDATENOW);

/** Global variables ********************************************************/

static HANDLE ghInstance;
static HWND hList1;
static COLORREF gaCustomColors[16]; // array of custom colors

void MainDlg_OnClose(HWND hwnd)
{
	EndDialog(hwnd, 0);
}

void MainDlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	static int RowHeight = 20;
	switch(id)
	{
		case IDC_GRIDLINES:
		{
			ListView_SetExtendedListViewStyle(hList1,LVS_EX_GRIDLINES ^
			ListView_GetExtendedListViewStyle(hList1));
		}
		break;

		case IDC_RESIZE_COL:
		{
			static BOOL fSet = FALSE;

			DataGridView_SetResizableHeader(hList1,fSet);
			fSet = !fSet;
		}
		break;

		case IDC_HIGHLIGHT_ROW:
		{
			ListView_SetExtendedListViewStyle(hList1,LVS_EX_FULLROWSELECT ^
			ListView_GetExtendedListViewStyle(hList1));
		}
		break;

		case IDC_COLOR_BK:
		{
			CHOOSECOLOR cc; 
		    memset(&cc, 0, sizeof(cc));
		    cc.lStructSize = sizeof(cc);
		    cc.hwndOwner = hwnd;
		    cc.hInstance = (HWND)ghInstance;
		    cc.rgbResult = ListView_GetBkColor(hList1);
		    cc.lpCustColors = (LPDWORD) gaCustomColors;
		    cc.Flags = CC_FULLOPEN | CC_RGBINIT;
			if (ChooseColor(&cc))
			{
				ListView_SetBkColor(hList1,cc.rgbResult);
				Refresh(hList1);
			}
		}
		break;

		case IDC_ALT_CLR_ROW:
		{
			CHOOSECOLOR cc; 
		    memset(&cc, 0, sizeof(cc));
		    cc.lStructSize = sizeof(cc);
		    cc.hwndOwner = hwnd;
		    cc.hInstance = (HWND)ghInstance;
		    cc.rgbResult = DataGridView_GetAltBkColor(hList1);
		    cc.lpCustColors = (LPDWORD) gaCustomColors;
		    cc.Flags = CC_FULLOPEN | CC_RGBINIT;
			if (ChooseColor(&cc))
			{
				DataGridView_SetAltBkColor(hList1,cc.rgbResult,TRUE); 
				Refresh(hList1);
			}
		}
		break;

		case IDC_ALT_CLR_COL:
		{
			CHOOSECOLOR cc; 
		    memset(&cc, 0, sizeof(cc));
		    cc.lStructSize = sizeof(cc);
		    cc.hwndOwner = hwnd;
		    cc.hInstance = (HWND)ghInstance;
		    cc.rgbResult = DataGridView_GetAltBkColor(hList1);
		    cc.lpCustColors = (LPDWORD) gaCustomColors;
		    cc.Flags = CC_FULLOPEN | CC_RGBINIT;
			if (ChooseColor(&cc))
			{
				DataGridView_SetAltBkColor(hList1,cc.rgbResult,FALSE); 
				Refresh(hList1);
			}
		}
		break;

		case IDC_TXT_CLR:
		{
			CHOOSECOLOR cc; 
		    memset(&cc, 0, sizeof(cc));
		    cc.lStructSize = sizeof(cc);
		    cc.hwndOwner = hwnd;
		    cc.hInstance = (HWND)ghInstance;
		    cc.rgbResult = ListView_GetTextColor(hList1);
		    cc.lpCustColors = (LPDWORD) gaCustomColors;
		    cc.Flags = CC_FULLOPEN | CC_RGBINIT;
			if (ChooseColor(&cc))
			{
				ListView_SetTextColor(hList1,cc.rgbResult); 
				Refresh(hList1);
			}
		}
		break;

		case IDC_ALT_TXT_CLR:
		{
			CHOOSECOLOR cc; 
		    memset(&cc, 0, sizeof(cc));
		    cc.lStructSize = sizeof(cc);
		    cc.hwndOwner = hwnd;
		    cc.hInstance = (HWND)ghInstance;
		    cc.rgbResult = DataGridView_GetAltTextColor(hList1);
		    cc.lpCustColors = (LPDWORD) gaCustomColors;
		    cc.Flags = CC_FULLOPEN | CC_RGBINIT;
			if (ChooseColor(&cc))
			{
				DataGridView_SetAltTextColor(hList1,cc.rgbResult); 
				Refresh(hList1);
			}
		}
		break;

		case IDC_DBLCLK:
		{
			static BOOL fDblClk = TRUE;
			DataGridView_SetDoubleClickSelect(hList1, fDblClk);
			fDblClk = !fDblClk;
		}
		break;

		case IDC_ROWHDR:
		{
			static BOOL fRwHdr = TRUE;
			DataGridView_FirstColumnRowHeaders(hList1, fRwHdr);
			fRwHdr = !fRwHdr;
		}
		break;

		case IDC_PLUS:
		{
			RowHeight++;
			DataGridView_SetRowHeight(hList1,RowHeight);
		}
		break;

		case IDC_MINUS:
		{
			RowHeight--;
			DataGridView_SetRowHeight(hList1,RowHeight);
		}
		break;
	}
	// TODO: Add your message processing code here...
}

void MainDlg_OnSize(HWND hwnd, UINT state, int cx, int cy)
{
	RECT rect;
	GetClientRect(hwnd,&rect);
	rect.left +=80;

	MoveWindow(hList1,
		rect.left, rect.top,
		rect.right-rect.left,
		rect.bottom-rect.top,
		TRUE);
}

BOOL MainDlg_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	hList1 = GetDlgItem(hwnd,IDC_GRID);

	//DataGridView_SetEditorBkColor(hList1,RGB(128,128,255));

	//Make some random data
	char aryByte[2048][3];//256
	srand(11);
	for(int i = 0; i < NELEMS(aryByte); ++i)
		sprintf(aryByte[i],"%02hhX",rand());

	//Create an array of Column header names
	char szHdr [17][3];
	for(int Col = -1; Col<17;)
		if(-1==Col) strcpy(szHdr[++Col],"  ");
		else sprintf(szHdr[++Col],"%02hhX",Col);

	// Add the columns based on header text
	DataGridView_AddColumns(hList1,szHdr,NELEMS(szHdr));

	// Add rows of random data
	char Items [17][256];
	for(int Row = 0, Col = 0, i = 0;
		(Col = Col < NELEMS(Items)? Col:0) , i < NELEMS(aryByte);)
	{
		// Col 0 will be the row number
		if(0==Col) sprintf(Items[Col++],"%02hhX",Row++);

		strcpy(Items[Col++],aryByte[i++]);

		if(Col == NELEMS(Items))
			DataGridView_AddRow(hList1,Items,NELEMS(Items));
		    
	}

	// Disable Auto Focus
	return FALSE;
}

/****************************************************************************
 *                                                                          *
 * Function: MainDlg_Proc                                                   *
 *                                                                          *
 * Purpose : Process messages for the Main dialog.                          *
 *                                                                          *
 * History : Date      Reason                                               *
 *           00/00/00  Created                                              *
 *                                                                          *
 ****************************************************************************/

BOOL CALLBACK MainDlg_Proc (HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		HANDLE_MSG (hwndDlg, WM_CLOSE, MainDlg_OnClose);
		HANDLE_MSG (hwndDlg, WM_COMMAND, MainDlg_OnCommand);
		HANDLE_MSG (hwndDlg, WM_INITDIALOG, MainDlg_OnInitDialog);
		HANDLE_MSG (hwndDlg, WM_SIZE, MainDlg_OnSize);

		//// TODO: Add dialog message crackers here...

		default: return FALSE;
	}
}

/****************************************************************************
 *                                                                          *
 * Function: WinMain                                                        *
 *                                                                          *
 * Purpose : Initialize the application.  Register a window class,          *
 *           create and display the main window and enter the               *
 *           message loop.                                                  *
 *                                                                          *
 * History : Date      Reason                                               *
 *           00/00/00  Created                                              *
 *                                                                          *
 ****************************************************************************/

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
    INITCOMMONCONTROLSEX icc;
    WNDCLASSEX wcx;

    ghInstance = hInstance;

    /* Initialize common controls. Also needed for MANIFEST's */

    icc.dwSize = sizeof(icc);
    icc.dwICC = ICC_WIN95_CLASSES /*|ICC_COOL_CLASSES|ICC_DATE_CLASSES|ICC_PAGESCROLLER_CLASS|ICC_USEREX_CLASSES*/;
    InitCommonControlsEx(&icc);

	InitDataGridView(hInstance);

	/* Load Rich Edit control support */
 
    // LoadLibrary(_T("riched32.dll"));  // Rich Edit v1.0
    // LoadLibrary(_T("riched20.dll"));  // Rich Edit v2.0, v3.0

    /* Get system dialog information */
    wcx.cbSize = sizeof(wcx);
    if (!GetClassInfoEx(NULL, MAKEINTRESOURCE(32770), &wcx))
        return 0;

    /* Add our own stuff */
    wcx.hInstance = hInstance;
    wcx.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDR_ICO_MAIN));
    wcx.lpszClassName = _T("SDKDataGClass");
    if (!RegisterClassEx(&wcx))
        return 0;

    /* The user interface is a modal dialog box */
    return DialogBox(hInstance, MAKEINTRESOURCE(DLG_MAIN), NULL, (DLGPROC)MainDlg_Proc);
}
